'
'     Copyright  2002-2003, The KPD-Team
'     All rights reserved.
'     http://www.mentalis.org/
'
'   Redistribution and use in source and binary forms, with or without
'   modification, are permitted provided that the following conditions
'   are met:
'
'     - Redistributions of source code must retain the above copyright
'        notice, this list of conditions and the following disclaimer. 
'
'     - Neither the name of the KPD-Team, nor the names of its contributors
'        may be used to endorse or promote products derived from this
'        software without specific prior written permission. 
'
'   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
'   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
'   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
'   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
'   THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
'   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
'   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
'   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
'   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
'   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
'   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
'   OF THE POSSIBILITY OF SUCH DAMAGE.
'

Imports System

namespace Org.Mentalis.Network 
	'/// <summary>
	'/// Parses a given string representation of a URL into its components and optionally reassembles these components back to a string representation.
	'/// </summary>
	public notinheritable class Url 
		'/// <summary>
		'/// Initializes a new Url instance.
		'/// </summary>
		'/// <param name="url">The URL.</param>
		'/// <exception cref="ArgumentNullException"><paramref name="url"/> is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		'/// <exception cref="ArgumentException"><paramref name="url"/> is invalid.</exception>
		public sub New(url as string ) 
			if url is nothing then throw new ArgumentNullException()
			' get protocol
			dim pos as integer = url.IndexOf("://", 0)
			if pos <= 0 then throw new ArgumentException("The specified URL is invalid.")
			Protocol = url.Substring(0, pos)
			url = url.Substring(pos + 3)
			' get query string
			pos = url.IndexOf("?")
			if pos < 0 then
				Query = ""
			else
				Query = url.Substring(pos + 1)
			    url = url.Substring(0, pos)
			end if
			' get host [+port/user/pass]
			dim parts() as string
			pos = url.IndexOf("/")
			if (pos < 0) then
				Path = ""
				parts = GetUrlParts(url)
			else
				Path = url.Substring(pos)
				parts = GetUrlParts(url.Substring(0, pos))
			end if
			Username = parts(0)
			Password = parts(1)
			Host = parts(2)
			try 
				Port = Integer.Parse(parts(3))
				If Port = 0 Then Port = GetDefaultPort(Protocol)
			catch e as Exception 
				throw new ArgumentException("The specified port is invalid.", e)
			end try
		end sub
		'/// <summary>
		'/// Returns a default port number for well known algorithms.
		'/// </summary>
		'/// <param name="protocol">The protocol.</param>
		'/// <returns>A number that corresponds to the default port of the given protocol -or- zero if the protocol is unknown.</returns>
		private function GetDefaultPort(protocol as string) as integer
			dim protocols() as string = new string() {"ftp", "ssh", "telnet", "smtp", "gopher", "http", "pop3", "nntp", "news", "https"}
			dim ports() as integer = new integer() {21, 22, 23, 25, 70, 80, 110, 119, 119, 443}
			dim index as integer = Array.IndexOf(protocols, protocol.ToLower(), 0, protocols.Length)
			if index = -1 then
				return 0 ' not a default protocol
			else
				return ports(index)
            end if
		end function
		'/// <summary>
		'/// Extracts the username, password, hostname and port from a partial URL.
		'/// </summary>
		'/// <param name="url">The partial URL to use.</param>
		'/// <returns>An array of Strings that contain the username, password, hostname and port (in that order).</returns>
		'/// <exception cref="ArgumentException">The specified URL is invalid.</exception>
		private function GetUrlParts(url as string)as string() 
			dim ret(3) as string 
			dim up as string, hp as string ' user/pass and host/port
			dim pos as integer = url.IndexOf("@")
			if pos < 0 then
				up = ""
				hp = url
			else
				up = url.Substring(0, pos)
				hp = url.Substring(pos + 1)
			end if
			' extract username/password
			pos = up.IndexOf(":")
			if pos < 0 then
				ret(0) = up
				ret(1) = ""
			else 
				ret(0) = up.Substring(0, pos)
				ret(1) = up.Substring(pos + 1)
			end if
			if ret(1) <> "" andalso ret(0) = "" then throw new ArgumentException("The specified URL is invalid.")
			' extract host/port
			pos = hp.IndexOf(":")
			if pos < 0 then
				ret(2) = hp
				ret(3) = "0"
			else 
				ret(2) = hp.Substring(0, pos)
				ret(3) = hp.Substring(pos + 1)
			end if
			if ret(2).Length = 0 then throw new ArgumentException("The specified URL is invalid.")
			return ret
		end function
		'/// <summary>
		'/// Holds the protocol of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the protocol of the URL.</value>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property Protocol as string 
			get 
				return m_Protocol
			end get
			set (Value as string)
				if Value is nothing then throw new ArgumentNullException()
				m_Protocol = value
			end set
		end property
		'/// <summary>
		'/// Holds the host of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the host of the URL.</value>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property Host as string 
			get 
				return m_Host
			end get
			set (Value as string)
				if value is nothing then throw new ArgumentNullException()
				m_Host = value
			end set
		end property
		'/// <summary>
		'/// Holds the port of the URL.
		'/// </summary>
		'/// <value>An <see cref="Int32"/> representing the port of the URL.</value>
		'/// <remarks>This value is equal to zero if no port is specified.</remarks>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property  Port as integer 
			get 
				return m_Port
			end get
			set (Value as integer)
				m_Port = value
			end set
		end property
		'/// <summary>
		'/// Holds the username of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the username of the URL.</value>
		'/// <remarks>This property can return an empty string.</remarks>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property Username as string
			get 
				return m_Username
			end get
			set (Value as string)
				if value is nothing then throw new ArgumentNullException()
				m_Username = value
			end set
		end property
		'/// <summary>
		'/// Holds the password of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the password of the URL.</value>
		'/// <remarks>This property can return an empty string.</remarks>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property Password as string
			get 
				return m_Password
			end get
			set (Value as string)
				if value is nothing then throw new ArgumentNullException()
				m_Password = value
			end set
		end property
		'/// <summary>
		'/// Holds the path of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the path of the URL.</value>
		'/// <remarks>This property can return an empty string.</remarks>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property  Path as string
			get 
				return m_Path
			end get
			set (Value as string)
				if value is nothing then throw new ArgumentNullException()
				m_Path = value
			end set
		end property
		'/// <summary>
		'/// Holds the query string of the URL.
		'/// </summary>
		'/// <value>A <see cref="String"/> representing the query string of the URL.</value>
		'/// <remarks>This property can return an empty string.</remarks>
		'/// <exception cref="ArgumentNullException">The specified value is a null reference (<b>Nothing</b> in Visual Basic).</exception>
		public property  Query as string
			get 
				return m_Query
			end get
			set (Value as string)
				if value is nothing then throw new ArgumentNullException()
				m_Query = value
			end set
		end property
		'/// <summary>
		'/// Returns a String that represents the current URL.
		'/// </summary>
		'/// <returns>Returns a String that represents the current URL.</returns>
		public overrides function ToString() as string
			dim user as string = "", host as string = Me.Host, query as string = ""
			if Username <> "" then user += Username
			if Password <> "" then user += ":" + Password
			if user <> "" then user += "@"
			if Port <> 0 then host += ":" + Port.ToString()
			if Me.Query <> "" then query += "?" + Me.Query
			return Protocol + "://" + user + host + Path + query
		end function
		'/// <summary>Holds the value of the Protocol property.</summary>
		private m_Protocol as string 
		'/// <summary>Holds the value of the Host property.</summary>
		private m_Host as string 
		'/// <summary>Holds the value of the Port property.</summary>
		private  m_Port as integer
		'/// <summary>Holds the value of the Username property.</summary>
		private m_Username as string 
		'/// <summary>Holds the value of the Password property.</summary>
		private m_Password as string 
		'/// <summary>Holds the value of the Path property.</summary>
		private m_Path as string 
		'/// <summary>Holds the value of the Query property.</summary>
		private m_Query as string 
	end class
end namespace